@@ -166,7 +166,7 @@ class SphinxComponentRegistry:
                 f"'env'argument. Report this bug to the developers of your custom builder, "
                 f"this is likely not a issue with Sphinx. The 'env' argument will be required "
                 f"from Sphinx 7.", RemovedInSphinx70Warning, stacklevel=2)
-            builder = self.builders[name](app, env=...)  # type: ignore[arg-type]
+            builder = self.builders[name](app)
             if env is not None:
                 builder.set_environment(env)
             return builder
